@@ -11,7 +11,7 @@ import { GraphTabContent } from './GraphTabContent';
 import DataTable from './DataTable';
 import TimeInput from './TimeInput';
 import QueryStatsView, { QueryStats } from './QueryStatsView';
-import { QueryParams } from '../../types/types';
+import { QueryParams, ExemplarData } from '../../types/types';
 import { API_PATH } from '../../constants/constants';
 
 interface PanelProps {
@@ -27,10 +27,13 @@ interface PanelProps {
   enableAutocomplete: boolean;
   enableHighlighting: boolean;
   enableLinter: boolean;
+  id: string;
 }
 
 interface PanelState {
+  // eslint-disable-next-line @typescript-eslint/no-explicit-any
   data: any; // TODO: Type data.
+  exemplars: ExemplarData;
   lastQueryParams: QueryParams | null;
   loading: boolean;
   warnings: string[] | null;
@@ -46,6 +49,7 @@ export interface PanelOptions {
   endTime: number | null; // Timestamp in milliseconds.
   resolution: number | null; // Resolution in seconds.
   stacked: boolean;
+  showExemplars: boolean;
 }
 
 export enum PanelType {
@@ -60,6 +64,7 @@ export const PanelDefaultOptions: PanelOptions = {
   endTime: null,
   resolution: null,
   stacked: false,
+  showExemplars: false,
 };
 
 class Panel extends Component<PanelProps, PanelState> {
@@ -70,6 +75,7 @@ class Panel extends Component<PanelProps, PanelState> {
 
     this.state = {
       data: null,
+      exemplars: [],
       lastQueryParams: null,
       loading: false,
       warnings: null,
@@ -79,23 +85,25 @@ class Panel extends Component<PanelProps, PanelState> {
     };
   }
 
-  componentDidUpdate({ options: prevOpts }: PanelProps) {
-    const { endTime, range, resolution, type } = this.props.options;
+  componentDidUpdate({ options: prevOpts }: PanelProps): void {
+    const { endTime, range, resolution, showExemplars, type } = this.props.options;
     if (
       prevOpts.endTime !== endTime ||
       prevOpts.range !== range ||
       prevOpts.resolution !== resolution ||
-      prevOpts.type !== type
+      prevOpts.type !== type ||
+      showExemplars !== prevOpts.showExemplars
     ) {
       this.executeQuery();
     }
   }
 
-  componentDidMount() {
+  componentDidMount(): void {
     this.executeQuery();
   }
 
-  executeQuery = (): void => {
+  // eslint-disable-next-line @typescript-eslint/no-explicit-any
+  executeQuery = async (): Promise<any> => {
     const { exprInputValue: expr } = this.state;
     const queryStart = Date.now();
     this.props.onExecuteQuery(expr);
@@ -138,58 +146,74 @@ class Panel extends Component<PanelProps, PanelState> {
         throw new Error('Invalid panel type "' + this.props.options.type + '"');
     }
 
-    fetch(`${this.props.pathPrefix}/${API_PATH}/${path}?${params}`, {
-      cache: 'no-store',
-      credentials: 'same-origin',
-      signal: abortController.signal,
-    })
-      .then(resp => resp.json())
-      .then(json => {
-        if (json.status !== 'success') {
-          throw new Error(json.error || 'invalid response JSON');
-        }
+    let query;
+    let exemplars;
+    try {
+      query = await fetch(`${this.props.pathPrefix}/${API_PATH}/${path}?${params}`, {
+        cache: 'no-store',
+        credentials: 'same-origin',
+        signal: abortController.signal,
+      }).then((resp) => resp.json());
 
-        let resultSeries = 0;
-        if (json.data) {
-          const { resultType, result } = json.data;
-          if (resultType === 'scalar') {
-            resultSeries = 1;
-          } else if (result && result.length > 0) {
-            resultSeries = result.length;
-          }
-        }
+      if (query.status !== 'success') {
+        throw new Error(query.error || 'invalid response JSON');
+      }
 
-        this.setState({
-          error: null,
-          data: json.data,
-          warnings: json.warnings,
-          lastQueryParams: {
-            startTime,
-            endTime,
-            resolution,
-          },
-          stats: {
-            loadTime: Date.now() - queryStart,
-            resolution,
-            resultSeries,
-          },
-          loading: false,
-        });
-        this.abortInFlightFetch = null;
-      })
-      .catch(error => {
-        if (error.name === 'AbortError') {
-          // Aborts are expected, don't show an error for them.
-          return;
+      if (this.props.options.type === 'graph' && this.props.options.showExemplars) {
+        params.delete('step'); // Not needed for this request.
+        exemplars = await fetch(`${this.props.pathPrefix}/${API_PATH}/query_exemplars?${params}`, {
+          cache: 'no-store',
+          credentials: 'same-origin',
+          signal: abortController.signal,
+        }).then((resp) => resp.json());
+
+        if (exemplars.status !== 'success') {
+          throw new Error(exemplars.error || 'invalid response JSON');
         }
-        this.setState({
-          error: 'Error executing query: ' + error.message,
-          loading: false,
-        });
+      }
+
+      let resultSeries = 0;
+      if (query.data) {
+        const { resultType, result } = query.data;
+        if (resultType === 'scalar') {
+          resultSeries = 1;
+        } else if (result && result.length > 0) {
+          resultSeries = result.length;
+        }
+      }
+
+      this.setState({
+        error: null,
+        data: query.data,
+        exemplars: exemplars?.data,
+        warnings: query.warnings,
+        lastQueryParams: {
+          startTime,
+          endTime,
+          resolution,
+        },
+        stats: {
+          loadTime: Date.now() - queryStart,
+          resolution,
+          resultSeries,
+        },
+        loading: false,
       });
+      this.abortInFlightFetch = null;
+    } catch (err: unknown) {
+      const error = err as Error;
+      if (error.name === 'AbortError') {
+        // Aborts are expected, don't show an error for them.
+        return;
+      }
+      this.setState({
+        error: 'Error executing query: ' + error.message,
+        loading: false,
+      });
+    }
   };
 
-  setOptions(opts: object): void {
+  setOptions(opts: Record<string, unknown>): void {
     const newOpts = { ...this.props.options, ...opts };
     this.props.onOptionsChanged(newOpts);
   }
@@ -209,15 +233,15 @@ class Panel extends Component<PanelProps, PanelState> {
     return this.props.options.endTime;
   };
 
-  handleChangeEndTime = (endTime: number | null) => {
+  handleChangeEndTime = (endTime: number | null): void => {
     this.setOptions({ endTime: endTime });
   };
 
-  handleChangeResolution = (resolution: number | null) => {
+  handleChangeResolution = (resolution: number | null): void => {
     this.setOptions({ resolution: resolution });
   };
 
-  handleChangeType = (type: PanelType) => {
+  handleChangeType = (type: PanelType): void => {
     if (this.props.options.type === type) {
       return;
     }
@@ -226,11 +250,19 @@ class Panel extends Component<PanelProps, PanelState> {
     this.setOptions({ type: type });
   };
 
-  handleChangeStacking = (stacked: boolean) => {
+  handleChangeStacking = (stacked: boolean): void => {
     this.setOptions({ stacked: stacked });
   };
 
-  render() {
+  handleChangeShowExemplars = (show: boolean): void => {
+    this.setOptions({ showExemplars: show });
+  };
+
+  handleTimeRangeSelection = (startTime: number, endTime: number): void => {
+    this.setOptions({ range: endTime - startTime, endTime: endTime });
+  };
+
+  render(): JSX.Element {
     const { pastQueries, metricNames, options } = this.props;
     return (
       <div className="panel">
@@ -316,16 +348,22 @@ class Panel extends Component<PanelProps, PanelState> {
                       useLocalTime={this.props.useLocalTime}
                       resolution={options.resolution}
                       stacked={options.stacked}
+                      showExemplars={options.showExemplars}
                       onChangeRange={this.handleChangeRange}
                       onChangeEndTime={this.handleChangeEndTime}
                       onChangeResolution={this.handleChangeResolution}
                       onChangeStacking={this.handleChangeStacking}
+                      onChangeShowExemplars={this.handleChangeShowExemplars}
                     />
                     <GraphTabContent
                       data={this.state.data}
+                      exemplars={this.state.exemplars}
                       stacked={options.stacked}
                       useLocalTime={this.props.useLocalTime}
+                      showExemplars={options.showExemplars}
                       lastQueryParams={this.state.lastQueryParams}
+                      id={this.props.id}
+                      handleTimeRangeSelection={this.handleTimeRangeSelection}
                     />
                   </>
                 )}
